# my_module.py

def greet(name):
    """ 사용자에게 인사말을 출력하는 함수 """
    return f"Hello, {name}!"

def add(a, b):
    """ 두 숫자의 합을 반환하는 함수 """
    return a + b
def remmove(a, b):
    """ 두 숫자의 차이를 반환하는 함수 """
    return a - b


from bs4 import BeautifulSoup
import requests

'''
 tensorflow API information
 - Document Address
 - GitHub Source Address
 - API Arguments
 - API Example code
 
 example:
 tf = tf()
 tf.get_info('tf.keras.layers.Conv2D')

 주의
 - tf.keras.- api , tf.applications.- api는 대부분 가능함
 - get_info에 String type으로 전달해야함

'''
class tf:

    def __init__(self):
        pass

    # get GitHub address
    def get_gitHub(self, soup):

        git = soup.find('table',{'class':'tfo-api'})
        if git is None:
            print("libaray is not available.")
            print("Right example: keras.callbacks.EarlyStopping")
            return False
        else:
            print("* Git Hub source *")
            result = git.find('a',{'target':'_blank'})['href']
            print(f"[GitHub]({result})")
    
    # get API arguments
    def get_args(self,soup):

        arg = soup.find('pre',{'class':'lang-py'})
        if arg is None:
            print("libaray is not available.")
            print("Right example: keras.layers.Cov2D")
        else:
          argument = arg.text.lstrip()
          print("* Arguments *")
          print(argument)

    # get Example code
    def get_ex(self,soup) -> None:

        print("* Code Examples *")
        code_tags = soup.find_all('code', {'class': 'devsite-terminal'})
        if code_tags is None:
            print("libaray is not available.")
            print("Right example: keras.optimizers.Adam")
        else:
            for code_tag in code_tags:
                code_text = code_tag.get_text(strip=True)
                print(">>> ",code_text)

    # get total information
    def get_info(self,input:str):
        assert type(input) == str , f"{input} must be a String type"
        # tensorflow API
        input = input.split('.')

        # tensorflow Document url
        for index,i in enumerate(input):
            if index == 0 and i == 'tf':
                url = f'https://www.tensorflow.org/api_docs/python/'
            elif index == 0 and i == 'keras':
                url = f'https://www.tensorflow.org/api_docs/python/tf/'
            url += i + '/'
        html = requests.get(url).text 
        soup = BeautifulSoup(html, 'html.parser')

        # print tensforflow API Info
        print("===============================================")
        print('* Document *')
        print(url)
        print(' ')
        self.get_gitHub(soup)
        print(' ')
        self.get_args(soup)
        self.get_ex(soup)
        print("===============================================")

# example usage
# x = tf()
# print(x.get_info('tf.keras.layers.Dropout'))
# print(x.get_info('keras.layers.Dropout'))



import matplotlib.pyplot as plt

def plot_history(history):
    # 학습 과정에서의 손실 값
    plt.figure(figsize=(12, 4))
    
    plt.subplot(1, 2, 1)
    plt.plot(history.history['loss'], label='Train Loss')
    plt.plot(history.history['val_loss'], label='Validation Loss')
    plt.title('Loss Over Epochs')
    plt.xlabel('Epoch')
    plt.ylabel('Loss')
    plt.legend()
    
    # 학습 과정에서의 정확도
    plt.subplot(1, 2, 2)
    plt.plot(history.history['accuracy'], label='Train Accuracy')
    plt.plot(history.history['val_accuracy'], label='Validation Accuracy')
    plt.title('Accuracy Over Epochs')
    plt.xlabel('Epoch')
    plt.ylabel('Accuracy')
    plt.legend()

    plt.show()



import json
from tensorflow.keras.callbacks import Callback
import matplotlib.pyplot as plt

class HistoryLogger(Callback):
    def __init__(self, log_file):
        self.log_file = log_file
        self.history = {}

    def on_epoch_end(self, epoch, logs=None):
        logs = logs or {}
        
        # 이전 로그 파일 읽기 (존재하면)
        previous_history = {}
        if os.path.isfile(self.log_file):
            with open(self.log_file, 'r') as f:
                previous_history = json.load(f)
        
        # 이전 에폭 값 가져오기
        previous_epochs = previous_history.get('epoch', [])
        if previous_epochs:
            last_epoch = previous_epochs[-1] + 1  # 마지막 에폭 값 + 1
        else:
            last_epoch = 0  # 이전 로그가 없으면 0부터 시작
        
        # 현재 에폭 번호 설정
        current_epoch = last_epoch + epoch
        
        # 현재 에폭 번호와 메트릭 저장
        self.history.setdefault('epoch', []).append(current_epoch)
        for k, v in logs.items():
            self.history.setdefault(k, []).append(v)

        # 로그 파일에 현재까지의 히스토리 저장
        with open(self.log_file, 'w') as f:
            json.dump(self.history, f)
def history_graph(history):
    # 손실 그래프 그리기
    plt.figure(figsize=(8, 4))
    plt.plot(history['epoch'], history['loss'], label='Training Loss')
    plt.plot(history['epoch'], history['val_loss'], label='Validation Loss')
    plt.title('Training and Validation Loss')
    plt.xlabel('Epoch')
    plt.ylabel('Loss')
    plt.legend()
    plt.show()

    # 정확도 그래프 그리기
    plt.figure(figsize=(8, 4))
    plt.plot(history['epoch'], history['accuracy'], label='Training Accuracy')
    plt.plot(history['epoch'], history['val_accuracy'], label='Validation Accuracy')
    plt.title('Training and Validation Accuracy')
    plt.xlabel('Epoch')
    plt.ylabel('Accuracy')
    plt.legend()
    plt.show()



from setuptools import setup, find_packages

setup(
    name='aiffel7',
    version='0.5.5',
    packages=find_packages(),
    description='A simple Python library',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author='aiffel7_그루',
    author_email='your.email@example.com',
    url='https://github.com/dudnjsckrgo/aiffel7_library',
    license='LICENSE',
    install_requires=[
        # 필요한 의존성 패키지 나열
        'beautifulsoup4',  # This is the package name for bs4
        'requests',
        'matplotlib',
        "tensorflow"
    ],
    classifiers=[
        # PyPI 분류자
        # 참조: https://pypi.org/classifiers/
    ],
)
